from __future__ import annotations

import pprint
from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Union, Any

if TYPE_CHECKING:
    from ..client import SyncPhraseTMSClient

from ..models import (
    AsyncRequestDto,
    PageDtoAsyncRequestDto,
    AsyncRequestStatusDto
    
)


class AsyncRequestOperations:
    def __init__(self, client: SyncPhraseTMSClient):
        self.client = client


    def get_async_request(
        self,
        async_request_id: int,
        phrase_token: Optional[str] = None,
) -> AsyncRequestDto:
        """
        Operation id: getAsyncRequest
        Get asynchronous request
        
This API call will return information about the specified
[asynchronous request](https://support.phrase.com/hc/en-us/articles/5709706916124-API-TMS-#asynchronous-apis-0-2).

Apart from basic information about the asynchronous operation such as who created it and for what action, the response
will contain a subset of [Get project](#operation/getProject) information.

The response contains an `asyncResponse` field which will remain `null` until the async request has finished processing.
If any errors occurred during processing of the request, this field will contain such errors or warnings.

_Note_: It is important to keep track of the number of pending asynchronous requests as these are subject to [Phrase
limits](https://support.phrase.com/hc/en-us/articles/5784117234972-Phrase-TMS-Limits#api-limits-async-requests-0-2).

        :param async_request_id: int (required), path. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AsyncRequestDto
        """
        endpoint = f"/api2/v1/async/{async_request_id}"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )
        jsn = r.json()

        return AsyncRequestDto(**jsn)
        


    def get_current_limit_status(
        self,
        phrase_token: Optional[str] = None,
) -> AsyncRequestStatusDto:
        """
        Operation id: getCurrentLimitStatus
        Get current limits
        
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AsyncRequestStatusDto
        """
        endpoint = f"/api2/v1/async/status"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AsyncRequestStatusDto(**r.json())
        


    def list_pending_requests(
        self,
        all: Optional[bool] = False,
        page_number: Optional[int] = 0,
        page_size: Optional[int] = 50,
        phrase_token: Optional[str] = None,
) -> PageDtoAsyncRequestDto:
        """
        Operation id: listPendingRequests
        List pending requests
        
API call to return a list of pending asynchronous requests.

Some operations within Phrase TMS are performed
[asynchronously](https://support.phrase.com/hc/en-us/articles/5784117234972-Phrase-TMS-Limits#api-limits-async-requests-0-2)
and their response only serves as an acknowledgement of receipt, not an actual completion of such request.
Since Phrase  imposes restrictions on the number of pending asynchronous
requests within an organization, this API call provides the means to check the number of such
pending requests.

When processing a large number of asynchronous operations, Phrase recommends periodically checking this list of
pending requests in order to not receive an error code during the actual processing of the requests.

_Note: Only actions triggered via the APIs are counted towards this limit, the same type of operation carried out via the
UI is not taken into account. This means that even with 200 pending requests, users can still create jobs via the UI._

        :param all: Optional[bool] = False (optional), query. Pending requests for organization instead of current user. Only for ADMIN..
        :param page_number: Optional[int] = 0 (optional), query. Page number, starting with 0, default 0.
        :param page_size: Optional[int] = 50 (optional), query. Page size, accepts values between 1 and 50, default 50.
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: PageDtoAsyncRequestDto
        """
        endpoint = f"/api2/v1/async"
        params = {
            "all": all,
            "pageNumber": page_number,
            "pageSize": page_size
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )
        dct = r.json()
        pprint.pp(dct)
        
        return PageDtoAsyncRequestDto(**dct)
        



if __name__ == '__main__':
    print("This module is not intended to be run directly.")


from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Union, Any

if TYPE_CHECKING:
    from ..client import SyncPhraseTMSClient

from ..models import (
    PageDtoVendorDto,
    CreateVendorDto,
    VendorDto,
    DeleteVendorsDto
    
)


class VendorOperations:
    def __init__(self, client: SyncPhraseTMSClient):
        self.client = client


    async def create_vendor(
        self,
        create_vendor_dto: CreateVendorDto,
        phrase_token: Optional[str] = None,
) -> VendorDto:
        """
        Operation id: createVendor
        Create vendor
        
        :param create_vendor_dto: CreateVendorDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: VendorDto
        """
        endpoint = f"/api2/v1/vendors"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = create_vendor_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return VendorDto(**r.json())
        


    async def delete_vendors(
        self,
        delete_vendors_dto: DeleteVendorsDto,
        phrase_token: Optional[str] = None,
) -> None:
        """
        Operation id: deleteVendors
        Delete vendors (batch)
        
        :param delete_vendors_dto: DeleteVendorsDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: None
        """
        endpoint = f"/api2/v1/vendors"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = delete_vendors_dto

        r = await self.client.make_request(
            "DELETE",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return
        


    async def get_vendor(
        self,
        vendor_uid: str,
        phrase_token: Optional[str] = None,
) -> VendorDto:
        """
        Operation id: getVendor
        Get vendor
        
        :param vendor_uid: str (required), path. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: VendorDto
        """
        endpoint = f"/api2/v1/vendors/{vendor_uid}"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return VendorDto(**r.json())
        


    async def list_vendors(
        self,
        name: Optional[str] = None,
        page_number: Optional[int] = 0,
        page_size: Optional[int] = 50,
        phrase_token: Optional[str] = None,
) -> PageDtoVendorDto:
        """
        Operation id: listVendors
        List vendors
        
        :param name: Optional[str] = None (optional), query. Name or the vendor, for filtering.
        :param page_number: Optional[int] = 0 (optional), query. Page number, starting with 0, default 0.
        :param page_size: Optional[int] = 50 (optional), query. Page size, accepts values between 1 and 50, default 50.
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: PageDtoVendorDto
        """
        endpoint = f"/api2/v1/vendors"
        params = {
            "pageNumber": page_number,
            "pageSize": page_size,
            "name": name
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return PageDtoVendorDto(**r.json())
        



if __name__ == '__main__':
    print("This module is not intended to be run directly.")


from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Union, Any

if TYPE_CHECKING:
    from ..client import SyncPhraseTMSClient

from ..models import (
    TaskMappingDto
    
)


class MappingOperations:
    def __init__(self, client: SyncPhraseTMSClient):
        self.client = client


    async def get_mapping_for_task(
        self,
        id: str,
        workflow_level: Optional[int] = 1,
        phrase_token: Optional[str] = None,
) -> TaskMappingDto:
        """
        Operation id: getMappingForTask
        Returns mapping for taskId (mxliff)
        
        :param id: str (required), path. 
        :param workflow_level: Optional[int] = 1 (optional), query. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: TaskMappingDto
        """
        endpoint = f"/api2/v1/mappings/tasks/{id}"
        params = {
            "workflowLevel": workflow_level
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return TaskMappingDto(**r.json())
        



if __name__ == '__main__':
    print("This module is not intended to be run directly.")


from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Union, Any

if TYPE_CHECKING:
    from ..client import SyncPhraseTMSClient

from ..models import (
    LoginUserDto,
    LoginResponseDto,
    LoginToSessionV3Dto,
    LoginWithGoogleDto,
    LoginDto,
    LoginToSessionResponseDto,
    LoginWithAppleDto,
    AppleTokenResponseDto,
    LoginToSessionResponseV3Dto,
    LoginResponseV3Dto,
    LoginToSessionDto,
    LoginOtherV3Dto,
    LoginV3Dto,
    LoginOtherDto
    
)


class AuthenticationOperations:
    def __init__(self, client: SyncPhraseTMSClient):
        self.client = client


    async def login(
        self,
        login_dto: LoginDto,
        phrase_token: Optional[str] = None,
) -> LoginResponseDto:
        """
        Operation id: login
        Login
        
        :param login_dto: LoginDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseDto
        """
        endpoint = f"/api2/v1/auth/login"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseDto(**r.json())
        


    async def login_by_apple_with_code(
        self,
        login_with_apple_dto: LoginWithAppleDto,
        native: Optional[bool] = None,
        phrase_token: Optional[str] = None,
) -> LoginResponseDto:
        """
        Operation id: loginByAppleWithCode
        Login with Apple with code
        
        :param login_with_apple_dto: LoginWithAppleDto (required), body. 
        :param native: Optional[bool] = None (optional), query. For sign in with code from native device.
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseDto
        """
        endpoint = f"/api2/v1/auth/loginWithApple/code"
        params = {
            "native": native
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_with_apple_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseDto(**r.json())
        


    async def login_by_apple_with_refresh_token(
        self,
        login_with_apple_dto: LoginWithAppleDto,
        phrase_token: Optional[str] = None,
) -> LoginResponseDto:
        """
        Operation id: loginByAppleWithRefreshToken
        Login with Apple refresh token
        
        :param login_with_apple_dto: LoginWithAppleDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseDto
        """
        endpoint = f"/api2/v1/auth/loginWithApple/refreshToken"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_with_apple_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseDto(**r.json())
        


    async def login_by_google(
        self,
        login_with_google_dto: LoginWithGoogleDto,
        phrase_token: Optional[str] = None,
) -> LoginResponseDto:
        """
        Operation id: loginByGoogle
        Login with Google
        
        :param login_with_google_dto: LoginWithGoogleDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseDto
        """
        endpoint = f"/api2/v1/auth/loginWithGoogle"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_with_google_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseDto(**r.json())
        


    async def login_other(
        self,
        login_other_dto: LoginOtherDto,
        phrase_token: Optional[str] = None,
) -> LoginResponseDto:
        """
        Operation id: loginOther
        Login as another user
        Available only for admin
        :param login_other_dto: LoginOtherDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseDto
        """
        endpoint = f"/api2/v1/auth/loginOther"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_other_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseDto(**r.json())
        


    async def login_other_v3(
        self,
        login_other_v3_dto: LoginOtherV3Dto,
        phrase_token: Optional[str] = None,
) -> LoginResponseV3Dto:
        """
        Operation id: loginOtherV3
        Login as another user
        Available only for admin
        :param login_other_v3_dto: LoginOtherV3Dto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseV3Dto
        """
        endpoint = f"/api2/v3/auth/loginOther"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_other_v3_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseV3Dto(**r.json())
        


    async def login_to_session(
        self,
        login_to_session_dto: LoginToSessionDto,
        phrase_token: Optional[str] = None,
) -> LoginToSessionResponseDto:
        """
        Operation id: loginToSession
        Login to session
        
        :param login_to_session_dto: LoginToSessionDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginToSessionResponseDto
        """
        endpoint = f"/api2/v1/auth/loginToSession"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_to_session_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginToSessionResponseDto(**r.json())
        


    async def login_to_session_2(
        self,
        login_to_session_v3_dto: LoginToSessionV3Dto,
        phrase_token: Optional[str] = None,
) -> LoginToSessionResponseV3Dto:
        """
        Operation id: loginToSession_2
        Login to session
        
        :param login_to_session_v3_dto: LoginToSessionV3Dto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginToSessionResponseV3Dto
        """
        endpoint = f"/api2/v3/auth/loginToSession"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_to_session_v3_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginToSessionResponseV3Dto(**r.json())
        


    async def login_v3(
        self,
        login_v3_dto: LoginV3Dto,
        phrase_token: Optional[str] = None,
) -> LoginResponseV3Dto:
        """
        Operation id: loginV3
        Login
        
        :param login_v3_dto: LoginV3Dto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginResponseV3Dto
        """
        endpoint = f"/api2/v3/auth/login"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = login_v3_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginResponseV3Dto(**r.json())
        


    async def logout(
        self,
        authorization: Optional[str] = None,
        token: Optional[str] = None,
        phrase_token: Optional[str] = None,
) -> None:
        """
        Operation id: logout
        Logout
        
        :param authorization: Optional[str] = None (optional), header. 
        :param token: Optional[str] = None (optional), query. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: None
        """
        endpoint = f"/api2/v1/auth/logout"
        params = {
            "token": token
            
        }
        headers = {
            "Authorization": authorization
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return
        


    async def refresh_apple_token(
        self,
        token: Optional[str] = None,
        phrase_token: Optional[str] = None,
) -> AppleTokenResponseDto:
        """
        Operation id: refreshAppleToken
        refresh apple token
        
        :param token: Optional[str] = None (optional), query. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AppleTokenResponseDto
        """
        endpoint = f"/api2/v1/auth/refreshAppleToken"
        params = {
            "token": token
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AppleTokenResponseDto(**r.json())
        


    async def who_am_i(
        self,
        phrase_token: Optional[str] = None,
) -> LoginUserDto:
        """
        Operation id: whoAmI
        Who am I
        
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: LoginUserDto
        """
        endpoint = f"/api2/v1/auth/whoAmI"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return LoginUserDto(**r.json())
        



if __name__ == '__main__':
    print("This module is not intended to be run directly.")


import logging
from typing import Optional, TypeVar, Dict, Any, Union

import httpx
from httpx import HTTPStatusError, Response
from httpx._types import RequestFiles
from pydantic import BaseModel

from .exceptions import PhrappyError
from .models import LoginDto
from .tags import (
    AdditionalWorkflowStepOperations,
    AnalysisOperations,
    AsyncRequestOperations,
    AuthenticationOperations,
    BilingualFileOperations,
    BusinessUnitOperations,
    BuyerOperations,
    ClientOperations,
    ConnectorOperations,
    CostCenterOperations,
    CustomFieldsOperations,
    CustomFileTypeOperations,
    DomainOperations,
    EmailTemplateOperations,
    FileOperations,
    GlossaryOperations,
    ImportsettingsOperations,
    ConversationsOperations,
    SupportedLanguagesOperations,
    LanguageQualityAssessmentOperations,
    QualityAssuranceOperations,
    MachineTranslationSettingsOperations,
    MachineTranslationOperations,
    MappingOperations,
    LanguageAIOperations,
    NetRateSchemeOperations,
    NotificationsOperations,
    PriceListOperations,
    ProjectTemplateOperations,
    TermBaseOperations,
    TranslationMemoryOperations,
    ProjectOperations,
    JobOperations,
    TranslationOperations,
    SegmentOperations,
    ProviderOperations,
    ProjectReferenceFileOperations,
    QuoteOperations,
    SCIMOperations,
    SegmentationRulesOperations,
    SpellCheckOperations,
    SubDomainOperations,
    UserOperations,
    WorkflowStepOperations,
    VendorOperations,
    WebhookOperations,
    XMLAssistantOperations,
    WorkflowchangesOperations
    
)


MEMSOURCE_BASE_URL = "https://cloud.memsource.com/web"

logger = logging.getLogger(__name__)

T = TypeVar('T', bound=BaseModel)


class Phrappy:
    """Client for interacting with the Phrase TMS API."""

    def __init__(self, token: Optional[str] = None, base_url: str = MEMSOURCE_BASE_URL):
        """Initialize the Phrase TMS client.
        
        Args:
            token: Optional API token. If provided without "ApiToken " prefix, it will be added.
            base_url: Base URL for API requests. Defaults to cloud.memsource.com.
        """
        self.base_url = base_url
        self.token = self._validate_token(token)
        self._init_operations()

    @staticmethod
    def _validate_token(token: Optional[str]) -> Optional[str]:
        """Validate and format the API token.
        
        Args:
            token: Raw API token string
            
        Returns:
            Properly formatted token or None
        """
        if not token:
            return None
        if token.startswith("ApiToken "):
            return token
        return f"ApiToken {token}"

    def _init_operations(self) -> None:
        """Initialize all API operation objects."""
        self.additional_workflow_step = AdditionalWorkflowStepOperations(self)
        self.analysis = AnalysisOperations(self)
        self.async_request = AsyncRequestOperations(self)
        self.authentication = AuthenticationOperations(self)
        self.bilingual_file = BilingualFileOperations(self)
        self.business_unit = BusinessUnitOperations(self)
        self.buyer = BuyerOperations(self)
        self.client = ClientOperations(self)
        self.connector = ConnectorOperations(self)
        self.cost_center = CostCenterOperations(self)
        self.custom_fields = CustomFieldsOperations(self)
        self.custom_file_type = CustomFileTypeOperations(self)
        self.domain = DomainOperations(self)
        self.email_template = EmailTemplateOperations(self)
        self.file = FileOperations(self)
        self.glossary = GlossaryOperations(self)
        self.importsettings = ImportsettingsOperations(self)
        self.conversations = ConversationsOperations(self)
        self.supported_languages = SupportedLanguagesOperations(self)
        self.language_quality_assessment = LanguageQualityAssessmentOperations(self)
        self.quality_assurance = QualityAssuranceOperations(self)
        self.machine_translation_settings = MachineTranslationSettingsOperations(self)
        self.machine_translation = MachineTranslationOperations(self)
        self.mapping = MappingOperations(self)
        self.language_ai = LanguageAIOperations(self)
        self.net_rate_scheme = NetRateSchemeOperations(self)
        self.notifications = NotificationsOperations(self)
        self.price_list = PriceListOperations(self)
        self.project_template = ProjectTemplateOperations(self)
        self.term_base = TermBaseOperations(self)
        self.translation_memory = TranslationMemoryOperations(self)
        self.project = ProjectOperations(self)
        self.job = JobOperations(self)
        self.translation = TranslationOperations(self)
        self.segment = SegmentOperations(self)
        self.provider = ProviderOperations(self)
        self.project_reference_file = ProjectReferenceFileOperations(self)
        self.quote = QuoteOperations(self)
        self.scim = SCIMOperations(self)
        self.segmentation_rules = SegmentationRulesOperations(self)
        self.spell_check = SpellCheckOperations(self)
        self.sub_domain = SubDomainOperations(self)
        self.user = UserOperations(self)
        self.workflow_step = WorkflowStepOperations(self)
        self.vendor = VendorOperations(self)
        self.webhook = WebhookOperations(self)
        self.xml_assistant = XMLAssistantOperations(self)
        self.workflowchanges = WorkflowchangesOperations(self)
        self._last_response: Optional[Response] = None

    def make_request(
        self,
        method: str,
        path: str,
        phrase_token: Optional[str] = None,
        params: Optional[Dict[str, Any]] = None,
        payload: Optional[Union[T, Dict[str, Any]]] = None,
        files: Optional[RequestFiles] = None,
        headers: Optional[Dict[str, str]] = None,
        content: Optional[bytes] = None,
        timeout: float = 180.0
    ) -> Response:
        """Make an HTTP request to the Phrase TMS API.

        Args:
            method: HTTP method (GET, POST, etc)
            path: API endpoint path
            phrase_token: Optional token override
            params: Optional URL parameters
            payload: Optional request body
            files: Optional files to upload
            headers: Optional HTTP headers
            content: Optional raw content body
            timeout: Request timeout in seconds

        Returns:
            httpx.Response object

        Raises:
            PhrappyError: If the API request fails
        """
        # Build request URL and headers
        if not path.startswith('/'):
            path = '/' + path
        url = self.base_url.rstrip('/') + path
        request_headers = {}
        if token := (phrase_token or self.token):
            request_headers["Authorization"] = token
        if headers:
            request_headers.update(headers)

        # Convert Pydantic models to dicts
        if payload is not None and not isinstance(payload, dict):
            try:
                payload = payload.model_dump(exclude_unset=True)
            except Exception as e:
                logger.exception("Failed to serialize payload")
                raise PhrappyError("Failed to serialize request payload") from e

        # Clean up None values from params
        if params:
            params = {k: v for k, v in params.items() if v is not None}
        response = None

        try:
            response = httpx.request(
                method=method,
                url=url,
                headers=request_headers,
                params=params,
                json=payload,
                files=files,
                content=content,
                timeout=timeout,
            )
            self._last_response = response
            response.raise_for_status()
            return response

        except HTTPStatusError as exc:
            if response:
                self._last_response = response
                try:
                    error_data = response.json()
                    error_code = error_data.get("errorCode")
                    error_desc = error_data.get("errorDescription")
                    msg = (
                        f"API request failed: {method} {url}\n"
                        f"Status: {response.status_code}\n"
                        f"Error code: {error_code}\n"
                        f"Description: {error_desc}"
                    )
                    raise PhrappyError(msg) from exc
                except ValueError:
                    raise PhrappyError(
                        f"API request failed with invalid JSON response: {response.text}"
                    ) from exc
            else:
                raise PhrappyError(f"Request failed with no response: {str(exc)}") from exc

        except Exception as e:
            logger.exception(f"Unexpected error during request: {url}")
            raise PhrappyError(f"An unexpected error occurred: {str(e)}") from e

    @classmethod
    def from_creds(cls, username: str, password: str) -> 'Phrappy':
        pp = cls()
        login_dto = {"userName": username, "password": password}
        resp = pp.authentication.login(login_dto)
        return cls(resp.token)



from typing import List, Optional, TYPE_CHECKING

from pydantic import BaseModel, Field

if TYPE_CHECKING:
    from .phrase_tms_api_models import UidReference, IdReference


class RateLimitsBaseModel(BaseModel):
    ratelimit_limit: Optional[int] = Field(alias="Ratelimit-Limit", default=None)
    ratelimit_remaining: Optional[int] = Field(
        alias="Ratelimit-Remaining", default=None
    )


class JobUpdateSourceMeta(BaseModel):
    jobs: List["UidReference"]
    preTranslate: bool = False
    allowAutomaticPostAnalysis: Optional[bool] = None
    callbackUrl: Optional[str] = None


class UploadHandoverFileMeta(BaseModel):
    jobs: List["UidReference"]


class JobUpdateTargetMeta(BaseModel):
    jobs: List["UidReference"] = Field(max_length=1)
    propagateConfirmedToTm: bool = True
    targetSegmentationRule: Optional["IdReference"] = None
    callbackUrl: Optional[str] = None
    unconfirmChangedSegments: Optional[bool] = True


class CreateSegmentationRuleMeta(BaseModel):
    name: str
    locale: str
    primary: bool = False
    filename: str


class UploadFileV2Meta(BaseModel):
    commitMessage: str
    subfolderName: str
    callbackUrl: Optional[str] = None



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, List, Optional, Union, Any

if TYPE_CHECKING:
    from ..client import SyncPhraseTMSClient

from ..models import (
    CreateAnalyseAsyncV2Dto,
    PageDtoAnalyseReference,
    AnalyseLanguagePartDto,
    AsyncAnalyseListResponseDto,
    AnalyseV3Dto,
    AnalyseV2Dto,
    AsyncAnalyseListResponseV2Dto,
    AnalyseJobDto,
    BulkEditAnalyseV2Dto,
    CreateAnalyseListAsyncDto,
    EditAnalyseV2Dto,
    AnalysesV2Dto,
    AnalyseRecalculateResponseDto,
    PageDtoAnalyseJobDto,
    AnalyseRecalculateRequestDto,
    BulkDeleteAnalyseDto
    
)


class AnalysisOperations:
    def __init__(self, client: SyncPhraseTMSClient):
        self.client = client


    async def analyses_batch_edit_v2(
        self,
        bulk_edit_analyse_v2_dto: BulkEditAnalyseV2Dto,
        phrase_token: Optional[str] = None,
) -> AnalysesV2Dto:
        """
        Operation id: analyses-batchEdit-v2
        Edit analyses (batch)
        If no netRateScheme is provided in request, then netRateScheme associated with provider will
be used if it exists, otherwise it will remain the same as it was.
        :param bulk_edit_analyse_v2_dto: BulkEditAnalyseV2Dto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AnalysesV2Dto
        """
        endpoint = f"/api2/v2/analyses/bulk"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = bulk_edit_analyse_v2_dto

        r = await self.client.make_request(
            "PUT",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AnalysesV2Dto(**r.json())
        


    async def bulk_delete_analyses(
        self,
        bulk_delete_analyse_dto: BulkDeleteAnalyseDto,
        phrase_token: Optional[str] = None,
) -> None:
        """
        Operation id: bulkDeleteAnalyses
        Delete analyses (batch)
        
        :param bulk_delete_analyse_dto: BulkDeleteAnalyseDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: None
        """
        endpoint = f"/api2/v1/analyses/bulk"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = bulk_delete_analyse_dto

        r = await self.client.make_request(
            "DELETE",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return
        


    async def create_analyse_async_v2(
        self,
        create_analyse_async_v2_dto: CreateAnalyseAsyncV2Dto,
        phrase_token: Optional[str] = None,
) -> AsyncAnalyseListResponseV2Dto:
        """
        Operation id: createAnalyseAsyncV2
        Create analysis
        Returns created analyses - batching analyses by number of segments (api.segment.count.approximation, default 100000), in case request contains more segments than maximum (api.segment.max.count, default 300000), returns 400 bad request.
        :param create_analyse_async_v2_dto: CreateAnalyseAsyncV2Dto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AsyncAnalyseListResponseV2Dto
        """
        endpoint = f"/api2/v2/analyses"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = create_analyse_async_v2_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AsyncAnalyseListResponseV2Dto(**r.json())
        


    async def create_analyses_for_langs(
        self,
        create_analyse_list_async_dto: CreateAnalyseListAsyncDto,
        phrase_token: Optional[str] = None,
) -> AsyncAnalyseListResponseDto:
        """
        Operation id: createAnalysesForLangs
        Create analyses by languages
        
        :param create_analyse_list_async_dto: CreateAnalyseListAsyncDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AsyncAnalyseListResponseDto
        """
        endpoint = f"/api2/v1/analyses/byLanguages"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = create_analyse_list_async_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AsyncAnalyseListResponseDto(**r.json())
        


    async def create_analyses_for_providers(
        self,
        create_analyse_list_async_dto: CreateAnalyseListAsyncDto,
        phrase_token: Optional[str] = None,
) -> AsyncAnalyseListResponseDto:
        """
        Operation id: createAnalysesForProviders
        Create analyses by providers
        
        :param create_analyse_list_async_dto: CreateAnalyseListAsyncDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AsyncAnalyseListResponseDto
        """
        endpoint = f"/api2/v1/analyses/byProviders"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = create_analyse_list_async_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AsyncAnalyseListResponseDto(**r.json())
        


    async def delete(
        self,
        analyse_uid: str,
        purge: Optional[bool] = None,
        phrase_token: Optional[str] = None,
) -> None:
        """
        Operation id: delete
        Delete analysis
        
        :param analyse_uid: str (required), path. 
        :param purge: Optional[bool] = None (optional), query. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: None
        """
        endpoint = f"/api2/v1/analyses/{analyse_uid}"
        params = {
            "purge": purge
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "DELETE",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return
        


    async def download_analyse(
        self,
        analyse_uid: str,
        format: str,
        phrase_token: Optional[str] = None,
) -> bytes:
        """
        Operation id: downloadAnalyse
        Download analysis
        
        :param analyse_uid: str (required), path. 
        :param format: str (required), query. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: bytes
        """
        endpoint = f"/api2/v1/analyses/{analyse_uid}/download"
        params = {
            "format": format
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return r.content
        


    async def edit_analysis(
        self,
        analyse_uid: str,
        edit_analyse_v2_dto: EditAnalyseV2Dto,
        phrase_token: Optional[str] = None,
) -> AnalyseV2Dto:
        """
        Operation id: editAnalysis
        Edit analysis
        If no netRateScheme is provided in
request, then netRateScheme associated with provider will be used if it exists, otherwise it will remain the same
as it was.
        :param analyse_uid: str (required), path. 
        :param edit_analyse_v2_dto: EditAnalyseV2Dto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AnalyseV2Dto
        """
        endpoint = f"/api2/v2/analyses/{analyse_uid}"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = edit_analyse_v2_dto

        r = await self.client.make_request(
            "PUT",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AnalyseV2Dto(**r.json())
        


    async def get_analyse_language_part(
        self,
        analyse_language_part_id: int,
        analyse_uid: str,
        phrase_token: Optional[str] = None,
) -> AnalyseLanguagePartDto:
        """
        Operation id: getAnalyseLanguagePart
        Get analysis language part
        Returns analysis language pair
        :param analyse_language_part_id: int (required), path. 
        :param analyse_uid: str (required), path. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AnalyseLanguagePartDto
        """
        endpoint = f"/api2/v1/analyses/{analyse_uid}/analyseLanguageParts/{analyse_language_part_id}"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AnalyseLanguagePartDto(**r.json())
        


    async def get_analyse_v3(
        self,
        analyse_uid: str,
        phrase_token: Optional[str] = None,
) -> AnalyseV3Dto:
        """
        Operation id: getAnalyseV3
        Get analysis
        
This API endpoint retrieves analysis results, encompassing basic information about the analysis, such as its name,
assigned provider,
[net rate scheme](https://support.phrase.com/hc/en-us/articles/5709665578908-Net-Rate-Schemes-TMS-),
[Analysis settings](https://support.phrase.com/hc/en-us/articles/5709712007708-Analysis-TMS-) settings and a subset of
[Get project](#operation/getProject) information for the project the analysis belongs to.

The analysis results consist of each analyzed language, presented as an item within the `analyseLanguageParts` array.
Each of these items contains details regarding the analyzed
[jobs](https://support.phrase.com/hc/en-us/articles/5709686763420-Jobs-TMS-),
[translation memories](https://support.phrase.com/hc/en-us/articles/5709688865692-Translation-Memories-Overview)
and the resultant data.

The analysis results are divided into two sections:

- `data` stores the raw numbers,
- `discountedData` recalculates the raw numbers using the selected net rate scheme.

Similar to the UI, both raw and net numbers are categorized based on their source into TM, MT, and NT categories,
including repetitions where applicable. These categories are then further subdivided based on the match score.

        :param analyse_uid: str (required), path. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AnalyseV3Dto
        """
        endpoint = f"/api2/v3/analyses/{analyse_uid}"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AnalyseV3Dto(**r.json())
        


    async def get_job_part_analyse(
        self,
        analyse_uid: str,
        job_uid: str,
        phrase_token: Optional[str] = None,
) -> AnalyseJobDto:
        """
        Operation id: getJobPartAnalyse
        Get jobs analysis
        Returns job's analyse
        :param analyse_uid: str (required), path. 
        :param job_uid: str (required), path. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AnalyseJobDto
        """
        endpoint = f"/api2/v1/analyses/{analyse_uid}/jobs/{job_uid}"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AnalyseJobDto(**r.json())
        


    async def list_by_project_v3(
        self,
        project_uid: str,
        name: Optional[str] = None,
        only_owner_org: Optional[bool] = None,
        order: Optional[str] = "desc",
        page_number: Optional[int] = 0,
        page_size: Optional[int] = 50,
        sort: Optional[str] = "DATE_CREATED",
        uid: Optional[str] = None,
        phrase_token: Optional[str] = None,
) -> PageDtoAnalyseReference:
        """
        Operation id: listByProjectV3
        List analyses by project
        
        :param project_uid: str (required), path. 
        :param name: Optional[str] = None (optional), query. Name to search by.
        :param only_owner_org: Optional[bool] = None (optional), query. 
        :param order: Optional[str] = "desc" (optional), query. Sorting order.
        :param page_number: Optional[int] = 0 (optional), query. 
        :param page_size: Optional[int] = 50 (optional), query. Page size, accepts values between 1 and 50, default 50.
        :param sort: Optional[str] = "DATE_CREATED" (optional), query. Sorting field.
        :param uid: Optional[str] = None (optional), query. Uid to search by.
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: PageDtoAnalyseReference
        """
        endpoint = f"/api2/v3/projects/{project_uid}/analyses"
        params = {
            "name": name,
            "uid": uid,
            "pageNumber": page_number,
            "pageSize": page_size,
            "sort": sort,
            "order": order,
            "onlyOwnerOrg": only_owner_org
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return PageDtoAnalyseReference(**r.json())
        


    async def list_job_parts(
        self,
        analyse_language_part_id: int,
        analyse_uid: str,
        page_number: Optional[int] = 0,
        page_size: Optional[int] = 50,
        phrase_token: Optional[str] = None,
) -> PageDtoAnalyseJobDto:
        """
        Operation id: listJobParts
        List jobs of analyses
        Returns list of job's analyses
        :param analyse_language_part_id: int (required), path. 
        :param analyse_uid: str (required), path. 
        :param page_number: Optional[int] = 0 (optional), query. Page number, starting with 0, default 0.
        :param page_size: Optional[int] = 50 (optional), query. Page size, accepts values between 1 and 50, default 50.
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: PageDtoAnalyseJobDto
        """
        endpoint = f"/api2/v1/analyses/{analyse_uid}/analyseLanguageParts/{analyse_language_part_id}/jobs"
        params = {
            "pageNumber": page_number,
            "pageSize": page_size
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return PageDtoAnalyseJobDto(**r.json())
        


    async def list_part_analyse_v3(
        self,
        job_uid: str,
        project_uid: str,
        page_number: Optional[int] = 0,
        page_size: Optional[int] = 50,
        phrase_token: Optional[str] = None,
) -> PageDtoAnalyseReference:
        """
        Operation id: listPartAnalyseV3
        List analyses
        
        :param job_uid: str (required), path. 
        :param project_uid: str (required), path. 
        :param page_number: Optional[int] = 0 (optional), query. 
        :param page_size: Optional[int] = 50 (optional), query. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: PageDtoAnalyseReference
        """
        endpoint = f"/api2/v3/projects/{project_uid}/jobs/{job_uid}/analyses"
        params = {
            "pageNumber": page_number,
            "pageSize": page_size
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = None

        r = await self.client.make_request(
            "GET",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return PageDtoAnalyseReference(**r.json())
        


    async def recalculate(
        self,
        analyse_recalculate_request_dto: AnalyseRecalculateRequestDto,
        phrase_token: Optional[str] = None,
) -> AnalyseRecalculateResponseDto:
        """
        Operation id: recalculate
        Recalculate analysis
        
        :param analyse_recalculate_request_dto: AnalyseRecalculateRequestDto (required), body. 
        
        :param phrase_token: string (optional) - if not supplied, client will look for token from init

        :return: AnalyseRecalculateResponseDto
        """
        endpoint = f"/api2/v1/analyses/recalculate"
        params = {
            
        }
        headers = {
            
        }

        content = None

        files = None

        payload = analyse_recalculate_request_dto

        r = await self.client.make_request(
            "POST",
            endpoint,
            phrase_token,
            params=params,
            payload=payload,
            files=files,
            headers=headers,
            content=content
        )

        
        return AnalyseRecalculateResponseDto(**r.json())
        



if __name__ == '__main__':
    print("This module is not intended to be run directly.")